import asyncio
import logging

import aiopg.sa
import uvloop
from aiogram import Bot, Dispatcher
from aiogram.contrib.fsm_storage.memory import MemoryStorage

from src.settings import settings

logging.basicConfig(level=logging.DEBUG,
                    format='%(asctime)s - %(name)s - %(levelname)s - %(message)s')


async def setup_db():
    engine = await aiopg.sa.create_engine(
        user=settings.postgres_user,
        password=settings.postgres_password,
        database=settings.postgres_database,
        port=settings.postgres_port,
        host=settings.postgres_host
    )
    settings.engine = engine


async def setup_bot():
    bot_params = {
        'token': settings.bot_token,
        'proxy': settings.proxy_pass
    }
    bot = Bot(**bot_params)
    storage = MemoryStorage()
    dispatcher = Dispatcher(bot, storage=storage)
    settings.dispatcher = dispatcher


async def run():
    await setup_db()
    await setup_bot()
    from src.actions import start_bot
    await start_bot()


if __name__ == '__main__':
    uvloop.install()
    fast_loop = asyncio.get_event_loop()
    fast_loop.run_until_complete(run())
