#!/usr/bin/env python
import logging
import os
import subprocess
import sys
import time

logger = logging.getLogger()


def file_filter(filename):
    return (not filename.startswith(".")) and (not filename.endswith(".swp"))


def venv_filter(walk_res):
    root, _dirs, _files = walk_res
    return not root.startswith('venv')


def file_times(root_path):
    for top_level in filter(file_filter, os.listdir(root_path)):
        for root, dirs, files in filter(venv_filter, os.walk(top_level)):
            for file in filter(file_filter, files):
                yield os.path.getmtime(os.path.join(root, file))


def print_stdout(running_process):
    stdout = running_process.stdout
    if stdout is not None:
        print(stdout)


if __name__ == "__main__":
    # We concatenate all of the arguments together, and treat that as the command to run
    command = ' '.join(sys.argv[1:])

    # The path to watch
    path = '.'

    # How often we check the filesystem for changes (in seconds)
    wait = 2

    # The process to watch
    process = subprocess.Popen(command, shell=True)

    # The current maximum file modified time under the watched directory
    last_mtime = max(file_times(path))
    logger.warning(last_mtime)
    while True:
        max_mtime = max(file_times(path))
        print_stdout(process)
        if max_mtime > last_mtime:
            logger.warning(last_mtime)
            last_mtime = max_mtime
            logger.warning(' CHANGES DETECTED RESTARTING '.center(100, '#'))
            process.kill()
            process = subprocess.Popen(command, shell=True)
        time.sleep(wait)
