from aiogram.types import Message, ParseMode

from src import __version__ as bot_version
from src.models.server import ServerPermissions
from src.settings import settings
from src.utils import chunks
from src.utils.decorators import (
    bot_action,
    BotParam
)
from src.utils.server_utils import get_server_by_alias
from src.utils.ssh import run_ssh_command


@settings.dispatcher.message_handler(commands=["help"])
async def help_command(message: Message):
    await settings.dispatcher.bot.send_message(message.chat.id,
                                               "Send `sys admin` to me to view full help.",
                                               ParseMode.MARKDOWN)


@bot_action(r'sys admin( help)?$')
async def overall_help(message: Message, state=None):
    await message.reply(
        '```\n'
        '* sys admin help - Show this message\n\n'
        '* sys admin servers - Manage your servers\n\n'
        '* sys admin permissions - Manage servers permissions\n\n'
        # '* sys admin commands - Manage stored commands\n\n'
        '* sys admin exec {alias} {command} - Execute single command\n\n'
        '* sys admin docker - Manage docker on remote server\n\n'
        '* sys admin sys - Manage systemctl on remote server\n\n'
        '* sys admin stats - See server statistics\n\n'
        '* sys admin interactive - Enter fully interactive mode\n\n'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(trigger_str='sys admin exec',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('command', r'(.*)')
            ])
async def bot_run_code(message: Message, alias: str, command: str, state=None):
    server = await get_server_by_alias(message, alias, ServerPermissions.RCE)
    results = chunks(await run_ssh_command(server, command), 4095)
    for res in results:
        await message.reply(f'```\n{res}```', parse_mode=ParseMode.MARKDOWN)


@bot_action(trigger_str='sys admin toggle debug')
async def debug_mode_toggle(message: Message, state=None):
    if settings.app_mode != 'dev':
        await message.reply('Debug mode is unavailable in production.')
        return
    if settings.debug_mode:
        settings.debug_mode = False
        await message.reply('Leaving debug mode')
    else:
        settings.debug_mode = True
        await message.reply('Entering debug mode')


@bot_action(trigger_str='sys admin info')
async def bot_info(message: Message, state):
    if settings.app_mode == 'prod':
        with open('app.info') as f:
            app_info = f.read()
    else:
        app_info = f'App mode: `{settings.app_mode}`'
    await message.reply(
        '* Le-memese system administrator info *\n\n'
        f'version: `{bot_version}`\n'
        f'{app_info}'
        , parse_mode=ParseMode.MARKDOWN
    )
