import json

from aiogram.types import Message, ParseMode

from src.models.server import ServerPermissions
from src.utils import chunks
from src.utils.decorators import (
    bot_action,
    BotParam
)
from src.utils.server_utils import get_server_by_alias
from src.utils.ssh import run_ssh_command


@bot_action(r'sys admin docker( help)?$')
async def overall_help(message: Message, state=None):
    await message.reply(
        '```\n'
        '* ls {alias} - List all running containers\n\n'
        '* clean {alias} - Remove exited containers\n\n'
        '* stop {alias} {containers} - stop one or more containers\n\n'
        '* rm {alias} {containers} - remove one or more containers\n\n'
        '* up {alias} {containers} - run one or more stopped containers\n\n'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(r'sys admin docker ls',
            params=[
                BotParam('alias', r'[\w\d]+')
            ])
async def docker_ls(message: Message, alias, state=None):
    server = await get_server_by_alias(message, alias, ServerPermissions.DOCKER)
    containers_ids_list = await run_ssh_command(server, 'docker ps -aq')
    if not containers_ids_list:
        await message.reply('* No running containers was found *', parse_mode=ParseMode.MARKDOWN)
        return
    ids = ' '.join(containers_ids_list.splitlines())
    response = await run_ssh_command(server, f'docker inspect {ids}')
    inspect_info = json.loads(response)
    formatted_containers = ""
    total = 0
    for container in inspect_info:
        total += 1
        formatted_containers += (
            '```\n'
            f'ID: {container["Id"][:12]}\n'
            f'name: {container["Name"]}\n'
            f'status: {container["State"]["Status"]}\n'
            f'networks: {", ".join(container["NetworkSettings"]["Networks"].keys())}\n'
            '```\n')
    message_text = ('* Docker containers: *\n'
                    f'*Total:* {total}\n\n'
                    f'{formatted_containers}')

    results = chunks(message_text, 4095)
    for res in results:
        await message.reply(res, parse_mode=ParseMode.MARKDOWN)


@bot_action(r'sys admin docker clean',
            params=[
                BotParam('alias', r'[\w\d]+')
            ])
async def docker_clean(message: Message, alias, state=None):
    server = await get_server_by_alias(message, alias, ServerPermissions.DOCKER)
    output = await run_ssh_command(server, 'docker ps -a | grep Exit | cut -d ' ' -f 1 | xargs docker rm')
    total = len(output.splitlines())
    await message.reply(
        '* Removed containers: *\n\n'
        f'*Total:* {total}'
        '```\n'
        f'{output}'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(r'sys admin docker rm',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('containers', r'.*')
            ])
async def docker_rm(message: Message, alias, containers, state=None):
    server = await get_server_by_alias(message, alias, ServerPermissions.DOCKER)
    output = await run_ssh_command(server, f'docker rm {containers}')
    total = len(output.splitlines())
    await message.reply(
        '* Removed containers: *\n\n'
        f'*Total:* {total}'
        '```\n'
        f'{output}'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(r'sys admin docker stop',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('containers', r'.*')
            ])
async def docker_rm(message: Message, alias, containers, state=None):
    server = await get_server_by_alias(message, alias, ServerPermissions.DOCKER)
    output = await run_ssh_command(server, f'docker stop {containers}')
    total = len(output.splitlines())
    await message.reply(
        '* Stopped containers: *\n\n'
        f'*Total:* {total}'
        '```\n'
        f'{output}'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(r'meme admin docker up',
            params=[
                BotParam('alias', r'[\w\d]+'),
                BotParam('containers', r'.*')
            ])
async def docker_rm(message: Message, alias, containers, state=None):
    server = await get_server_by_alias(message, alias, ServerPermissions.DOCKER)
    output = await run_ssh_command(server, f'docker start {containers}')
    total = len(output.splitlines())
    await message.reply(
        '* Upped containers: *\n\n'
        f'*Total:* {total}'
        '```\n'
        f'{output}'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )
