import logging

from aiogram.types import Message, ParseMode

from src.models.crud.server_crud import fn_update_server
from src.models.server import ServerPermissions
from src.settings import settings
from src.utils.debug_mode import debug_message
from src.utils.decorators import (
    bot_action,
    BotParam
)
from src.utils.server_utils import get_server_by_alias

logger = logging.getLogger(__name__)


@bot_action(r'sys admin permissions( help)?$')
async def permissions_help(message: Message, state=None):
    await message.reply(
        '```\n'
        '* ls - list all available permissions with description\n\n'
        '* grant {alias} {permission name} - grant specific permissions level to this server\n'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action('sys admin permissions ls')
async def list_permissions(message: Message, state=None):
    permissions = '\n\n'.join([f'* {perm.name}: {perm.__doc__}' for perm in ServerPermissions])
    await message.reply(f'Available permissions levels:\n ```\n{permissions}```',
                        parse_mode=ParseMode.MARKDOWN)


@bot_action(trigger_str=r'sys admin permissions grant',
            params=[
                BotParam('server_alias', r'[\w\d]+'),
                BotParam('permission', r'\w+')
            ])
async def change_permissions(message: Message, server_alias, permission, state=None):
    server = await get_server_by_alias(message, server_alias)
    await debug_message(message,
                        f'Chat id: {message.chat.id}\n'
                        f'Server alias: {server_alias}\n'
                        f'Requested permission: {permission}\n'
                        f'Server admin: {server.server_admin}\n'
                        f'Current user: {str(message.from_user.id)}\n'
                        )

    if not (str(message.from_user.id) == server.server_admin):
        raise Exception('You have no permissions to change server permissions.\nOnly admins can do this.')
    if permission.upper() not in ServerPermissions.__members__:
        raise Exception(f'Permission level {permission} was not found.')
    new_permission = ServerPermissions[permission.upper()]
    await fn_update_server(engine=settings.engine,
                           chat_id=str(message.chat.id),
                           server_alias=server_alias,
                           new_server_permissions=new_permission)
    await message.reply(f'Permissions for `{server_alias}` updated to {permission}',
                        parse_mode=ParseMode.MARKDOWN)
