import logging

from aiogram.types import Message, ParseMode

from src.utils.debug_mode import debug_message
from src.utils.decorators import (
    bot_action,
    BotParam
)
from src.utils.server_utils import get_server_by_alias
from src.utils.ssh import run_ssh_command

logger = logging.getLogger(__name__)


@bot_action(r'sys admin stats( help)?$')
async def overall_help(message: Message, state=None):
    await message.reply(
        '```\n'
        '* ram {alias} - Show server RAM usage\n\n'
        '* mem {alias} - Show server memory usage\n\n'
        '* cpu {alias} - Show cpu usage\n\n'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(r'sys admin stats cpu',
            params=[
                BotParam('alias', r'[\w\d]+')
            ])
async def cpu_usage(message: Message, alias: str, state=None):
    server = await get_server_by_alias(message, alias)
    cpu_stats = await run_ssh_command(server, "grep 'cpu' /proc/stat")
    cores = cpu_stats.splitlines()
    await debug_message(message,
                        f'Chat id: {message.chat.id}\n'
                        f'Current user: {str(message.from_user.id)}\n'
                        f'Server cpu info: \n'
                        f'{cpu_stats}'
                        )

    def calculate_load(core_stats: str):
        logger.warning(core_stats)
        stats = core_stats.split()
        cpu_name = stats.pop(0)
        if cpu_name == 'cpu':
            cpu_name = 'Overall loading'
        else:
            cpu_name = f'core #{cpu_name.replace("cpu", "")}'
        stats = list(map(int, stats))
        usage = ((stats[0] + stats[2]) * 100) / (stats[0] + stats[2] + stats[3])
        return f'{cpu_name}: {usage:.2f}%'

    core_info = '\n'.join(map(calculate_load, cores))

    await message.reply(
        '* Current cpu load: *\n\n'
        '```\n'
        f'{core_info}'
        '```',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(r'sys admin stats ram',
            params=[
                BotParam('alias', r'[\w\d]+')
            ])
async def ram_usage(message: Message, alias: str, state=None):
    server = await get_server_by_alias(message, alias)
    ram_stats = await run_ssh_command(server, "free -m")
    table_head, *memories = ram_stats.splitlines()
    await debug_message(message,
                        f'Chat id: {message.chat.id}\n'
                        f'Current user: {str(message.from_user.id)}\n'
                        f'Server RAM info: \n'
                        f'{ram_stats}'
                        )

    def parse_stats(memory_stat: str):
        m_split = memory_stat.split()
        return (f'{m_split[0]}: \n'
                f'total: {m_split[1]} MB\n'
                f'used: {m_split[2]} MB\n'
                f'free: {m_split[3]} MB\n')

    stats = '\n'.join(map(parse_stats, memories))
    await message.reply(
        '* Current RAM usage: *\n\n'
        '```\n'
        f'{stats}'
        '```\n',
        parse_mode=ParseMode.MARKDOWN
    )


@bot_action(r'sys admin stats mem',
            params=[
                BotParam('alias', r'[\w\d]+')
            ])
async def mem(message: Message, alias: str, state=None):
    server = await get_server_by_alias(message, alias)
    ram_stats = await run_ssh_command(server, "df -hl /")
    table_head, info = ram_stats.splitlines()
    info = info.split()
    await debug_message(message,
                        f'Chat id: {message.chat.id}\n'
                        f'Current user: {str(message.from_user.id)}\n'
                        f'Server disk info: \n'
                        f'{ram_stats}'
                        )

    await message.reply(
        '* Current memory usage: *\n\n'
        '```\n'
        f'Total size: {info[1]}\n'
        f'Used: {info[2]}\n'
        f'Available: {info[3]}\n'
        f'Usage: {info[4]}\n'
        '```\n',
        parse_mode=ParseMode.MARKDOWN
    )
