import sqlalchemy as sa
from sqlalchemy import UniqueConstraint

from src.migrations import Base
from src.utils.enum_utils import AutoEnum


class ServerPermissions(AutoEnum):
    INFO = "Users can read server info"
    SCP = "Users can download and upload files to server"
    DOCKER = "Users  can manage docker daemon"
    SERVICES = "Users can manage systemctl services"
    RCE = "Users can execute remote commands on this server"
    DELETE = "Users can delete this server from list"


class Server(Base):
    __tablename__ = 'servers'
    chat_id = sa.Column(sa.String(), primary_key=True, index=True, nullable=False)
    server_admin = sa.Column(sa.String(), nullable=False)
    server_address = sa.Column(sa.String(length=100), primary_key=True, nullable=False)
    server_alias = sa.Column(sa.String(length=100), nullable=False)
    server_port = sa.Column(sa.Integer(), server_default='22', nullable=False)
    server_permissions = sa.Column(sa.Enum(ServerPermissions),
                                   server_default=sa.literal(ServerPermissions.INFO.name),
                                   nullable=False)
    __table_args__ = (UniqueConstraint('chat_id', 'server_alias', name='_chat_server_uc'),)
