from aiogram.types import Message

from src.models import Server
from src.models.crud.server_crud import fn_get_server
from src.models.server import ServerPermissions
from src.settings import settings


async def get_server_by_alias(message: Message,
                              server_alias: str,
                              minimal_permission: ServerPermissions = ServerPermissions.INFO) -> Server:
    found_servers = await fn_get_server(settings.engine,
                                        chat_id=str(message.chat.id),
                                        server_alias=server_alias)
    if not found_servers:
        all_chat_servers = await fn_get_server(settings.engine,
                                               chat_id=str(message.chat.id))
        if all_chat_servers:
            servers_names = '\n'.join(map(lambda s: f'* {s.server_alias} - {s.server_address}', all_chat_servers))
            raise Exception(
                f'Server `{server_alias}` is not linked to this chat. Available servers: \n\n{servers_names}'
            )
        else:
            raise Exception('No server linked to this chat.')
    server = found_servers[0]
    if server.server_permissions.value < minimal_permission.value \
            and str(message.from_user.id) != server.server_admin:
        raise Exception('You do not have permission for that action.\n'
                        f'Minimal permission level is `{minimal_permission.name}`\n'
                        f'Contact server administrator to change server permissions.')
    return server
